/* Copyright (C) 2008 Patrick Galbraith, Brian Aker

  See COPYING file found with distribution for license.

*/

#include <mysql.h>
#include <string.h>

#include <stdio.h>

#include <curl/curl.h>
#include "common.h"

my_bool http_get_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
char *http_get(UDF_INIT *initid, UDF_ARGS *args, char *result, unsigned long *length, char *is_null, char *error);
void http_get_deinit(UDF_INIT *initid);


static void *myrealloc(void *ptr, size_t size)
{
  /* There might be a realloc() out there that doesn't like reallocing
     NULL pointers, so we take care of it here */
  if(ptr)
    return realloc(ptr, size);
  else
    return malloc(size);
}

static size_t
result_cb(void *ptr, size_t size, size_t nmemb, void *data)
{
  size_t realsize= size * nmemb;
  struct st_results *res= (struct st_results *)data;

  res->result= (char *)myrealloc(res->result, res->size + realsize + 1);
  if (res->result)
  {
    memcpy(&(res->result[res->size]), ptr, realsize);
    res->size += realsize;
    res->result[res->size]= 0;
  }
  return realsize;
}

my_bool http_get_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
{
  unsigned int x;
  curl_function_st *container;

  /* this is how to fail */
  if (args->arg_count != 1)
  {
    strncpy(message,
            "one argument must be supplied: http_get('<url>').",
            MYSQL_ERRMSG_SIZE);
    return 1;
  }

  args->arg_type[0]= STRING_RESULT;

  initid->max_length= CURL_UDF_MAX_SIZE;
  container= (curl_function_st *)malloc(sizeof(curl_function_st));

  curl_global_init(CURL_GLOBAL_ALL);

  initid->ptr= (char *)container;

  return 0;
}
/*
  httpd_get
  get url, takes url arg
*/
char *http_get(UDF_INIT *initid, UDF_ARGS *args,
                __attribute__ ((unused)) char *result,
               unsigned long *length,
                __attribute__ ((unused)) char *is_null,
                __attribute__ ((unused)) char *error)
{
  CURLcode res;
  struct st_results chunk;

  chunk.result= NULL;
  chunk.size= 0;


  curl_function_st *container= (curl_function_st *)initid->ptr;
  container->curl= curl_easy_init();

  if (container->curl)
  {
    curl_easy_setopt(container->curl, CURLOPT_URL, args->args[0]);
    curl_easy_setopt(container->curl, CURLOPT_WRITEFUNCTION, result_cb);
    curl_easy_setopt(container->curl, CURLOPT_WRITEDATA, (void *)&chunk);
    curl_easy_setopt(container->curl, CURLOPT_USERAGENT, "libcurl-agent/1.0");
    res= curl_easy_perform(container->curl);
    return ((char*) &chunk.result);
  }
  else
  {
    return((char *)NULL);
  }
}

/* de-init UDF */
void http_get_deinit(UDF_INIT *initid)
{
  /* if we allocated initid->ptr, free it here */
  curl_function_st *container= (curl_function_st *)initid->ptr;

  curl_easy_cleanup(container->curl);
  free(container);
  return;
}
